@@ -1,5 +1,13 @@
 <?php
 
+/**
+  * SquirrelMail Virtual Host Login Plugin
+  * Copyright (C) 2003, 2004 Paul Lesneiwski <pdontthink@angrynerds.com>
+  * This program is licensed under GPL. See COPYING for details
+  *
+  */
+
+
 // include compatibility plugin
 //
 if (defined('SM_PATH'))
@@ -10,13 +18,21 @@ else if (file_exists('./plugins/compatibility/functions.php'))
    include_once('./plugins/compatibility/functions.php');
 
 
+/**
+  * Override Default Config Values
+  *
+  * This function looks up new configuration settins for the current
+  * user and replaces the default settings with those.  The new values
+  * might be retrieved from the virtual domain table (based on the
+  * hostname portion of the current URL), a per user file, etc.
+  *
+  */
 function overrideSmConfig() {
 
    global $allVirtualDomainsAreUnderOneSSLHost, 
           $smHostIsDomainThatUserLoggedInWith, $virtualDomainDataDir, 
           $data_dir, $domain, $virtualDomains, $plugins,  
-          $squirrelmail_plugin_hooks, $useSessionBased, 
-          $lockoutEnabled, $lockoutTable;
+          $squirrelmail_plugin_hooks, $useSessionBased;
       
   
   // get global variable for versions of PHP < 4.1 
@@ -34,10 +50,7 @@ function overrideSmConfig() {
   compatibility_sqsession_is_active();
 
 
-  if (compatibility_check_sm_version(1, 3))
-     include_once (SM_PATH . 'plugins/vlogin/data/config.php');
-  else
-     include_once ('../plugins/vlogin/data/config.php');
+  include_once (SM_PATH . 'plugins/vlogin/data/config.php');
 
 
   if ($useSessionBased)
@@ -52,7 +65,10 @@ function overrideSmConfig() {
 
   // grab hostname into local var
   //
-  $hostname = $_SERVER['HTTP_HOST'];
+  if (isset($_SERVER['HTTP_X_FORWARDED_HOST']) && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
+     $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
+  else
+     $hostname = $_SERVER['HTTP_HOST'];
   $hostname_stripped = deconstructDomainName($hostname);
 
 
@@ -98,6 +114,12 @@ function overrideSmConfig() {
   $firstTime = 1;
 
 
+  // allow inclusion of external configuration files for each domain
+  //
+  if (file_exists(SM_PATH . 'plugins/vlogin/data/domains/' . $hostname_stripped . '.vlogin.config.php'))
+    include_once(SM_PATH . 'plugins/vlogin/data/domains/' . $hostname_stripped . '.vlogin.config.php');
+
+
   // override the org_logo and other stuff if we find a match
   //
 // NOTE: the following line will solve PHP 4.3 problems when using
@@ -126,25 +148,6 @@ function overrideSmConfig() {
       foreach ($virtualDomains[$virtualDomain] as $setting => $value)
       {
 
-        // Look for locked out domains, redirect immediately if found
-        //
-        if ($lockoutEnabled)
-        { 
-           if (array_key_exists('###LOCKED OUT###', $virtualDomains[$virtualDomain])
-              && ($virtualDomains[$virtualDomain]['###LOCKED OUT###']))
-           {
-              sqsession_destroy();
-              header('Location: ../plugins/vlogin/' 
-                 . $virtualDomains[$virtualDomain]['###LOCKED OUT###']);
-              exit;
-           }
-           else if ($lockoutTable)
-           {
-              check_lockouts($lockoutTable, '', $hostname);
-           }
-        }
-
-
         // Go ahead and replace the string ###VIRTUAL_DOMAIN###
         //
         if (!is_array($value))
@@ -252,13 +255,6 @@ function overrideSmConfig() {
       }
 
       
-      if (in_array('data_dir', array_keys($virtualDomains[$virtualDomain]))) {
-          if ($useSessionBased)
-             $config_override['data_dir'] = $virtualDomains[$virtualDomain]['data_dir'];
-          else
-             $data_dir = $virtualDomains[$virtualDomain]['data_dir'];
-      }
-
       // exit this loop, unless this is the global default
       // 
       if( $virtualDomain != '*' ) break;
@@ -277,10 +273,7 @@ function overrideSmConfig() {
 
      compatibility_sqsession_register($config_override, 'config_override');
 
-     if (compatibility_check_sm_version(1, 3))
-        include_once (SM_PATH . 'plugins/multilogin/functions.php');
-     else
-        include_once ('../plugins/multilogin/functions.php');
+     include_once (SM_PATH . 'plugins/multilogin/functions.php');
 
      multilogin_sqoverride_config_do();
 
@@ -302,15 +295,24 @@ function overrideSmConfig() {
 
 
 
-// Magic happens below.
-function vlogin_domain_do() {
-
+/** 
+  * Remap Username When Logging In
+  * 
+  * Here is the meat of the vlogin magic.  It is used to figure
+  * out what the username should really be, possibly based on
+  * the hostname portion of the current URL, a sendmail style
+  * virtual users table, any number of different combinations
+  * of stripping parts of the hostname, etc.
+  *
+  */
+function vlogin_domain_do() 
+{
 
   global $plugins, $login_username, $$login_username, $debug, $data_dir, 
          $foundLoginAlias, $at, $dot, $dontUseHostName, $atConversion,
          $sendmailVirtualUserTable, $putHostNameOnFrontOfUsername,
          $allVirtualDomainsAreUnderOneSSLHost, $prefs_dsn, $useSessionBased,
-         $virtualDomains, $lockoutEnabled, $lockoutTable;
+         $virtualDomains, $removeDomainIfGiven, $alwaysAddHostName;
 
 
   // figure out where prefs are stored
@@ -331,7 +333,10 @@ function vlogin_domain_do() {
 
   // grab hostname into local var
   //
-  $hostname = $_SERVER['HTTP_HOST'];
+  if (isset($_SERVER['HTTP_X_FORWARDED_HOST']) && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
+     $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
+  else
+     $hostname = $_SERVER['HTTP_HOST'];
 
 
   // for sites where virtual host is pegged on the end of the main
@@ -346,26 +351,39 @@ function vlogin_domain_do() {
   }
 
 
-  if (compatibility_check_sm_version(1, 3))
-     include_once (SM_PATH . 'plugins/vlogin/data/config.php');
-  else
-     include_once ('../plugins/vlogin/data/config.php');
+  // get vlogin config settings
+  //
+  include_once (SM_PATH . 'plugins/vlogin/data/config.php');
 
 
 
-  $user=$login_username;
+  // what username did the user submit to us?
+  //
+  $user = $login_username;
   
+
   // if password_forget is loaded, use the obfuscated name
-  if (in_array('password_forget',$plugins)) {
+  //
+  if (in_array('password_forget', $plugins)) {
     if (!isset($$login_username)) compatibility_sqextractGlobalVar($login_username);
     if ($$login_username != '')
-      $user=$$login_username;
+      $user = $$login_username;
+  }
+
+
+  // if user logged in with full email address, 
+  // chop off domain info if needed
+  //
+  if ( $removeDomainIfGiven && strpos($user, $at) !== FALSE ) 
+  {
+     $user = substr($user, 0, strpos($user, $at));
   }
 
 
   // check for login_alias plugin
   //
-  if (in_array('login_alias',$plugins)) {
+  if (in_array('login_alias',$plugins)) 
+  {
 
 
     // check if login alias was already processed
@@ -374,26 +392,32 @@ function vlogin_domain_do() {
       if ($foundLoginAlias) return;
 
     }
-    else {
+    else 
+    {
 
       // check for login alias here and return if found
       // (but only if it is in the domain being used
       // to log in when dontUseHostName is off)
       //
       $filename = $data_dir.'login_alias.pref';
-      if ((!$prefsInDB && file_exists($filename)) || $prefsInDB) {
-         $loginAlias=getPref($data_dir,'login_alias',$user);
+      if ((!$prefsInDB && file_exists($filename)) || $prefsInDB) 
+      {
+        $loginAlias=getPref($data_dir,'login_alias',$user);
+
+        if (!empty($loginAlias))
+        {
+          if (!$dontUseHostName)
+          {
+            if (strpos($loginAlias, deconstructDomainName($hostname)) !== FALSE)
+              return;
+          }
+          else
+          {
+            return;
+          }
+
+        }
 
-         if (!empty($loginAlias))
-         {
-            if (!$dontUseHostName)
-            {
-               if (strpos($loginAlias, deconstructDomainName($hostname)) !== FALSE)
-                  return;
-            }
-            else
-               return;
-         }
       }
 
     }
@@ -401,8 +425,6 @@ function vlogin_domain_do() {
   }
 
 
-  // find domain name.  Only truly important bit.
-
 
   // convert "at" sign...
   //
@@ -410,10 +432,67 @@ function vlogin_domain_do() {
      $user = str_replace($otherAt, $at, $user);
 
 
-  // check and see if they decided to insert the host anyways
+
+/* what was I thinking... 
+
+  // for those sysadmins who like to torture their 
+  // users and require a domain as part of the login...
+  //
+  if( $forceDomainInUsername )
+  {
+
+    if ( strpos($user, $at) === FALSE 
+      || substr($user, strpos($user, $at) + strlen($at)) != deconstructDomainName($hostname))
+    {
+      logout_error(_("Username must include domain name") . ' ' . deconstructDomainName($hostname));
+      exit;
+    }
+
+  }
+
+
+
+  // rare cases where we want to strip off domain from
+  // username after everything has been done above
+  //
+  if ( $removeDomainAfterDomainCheck && strpos($user, $at) !== FALSE ) 
+  {
+     $user = substr($user, 0, strpos($user, $at));
+  }
+
+   $forceDomainInUsername
+   ----------------------
+   can be used to require the user to input the domain portion of
+   their username (which MUST match the hostname as reconstructed
+   using the settings you have made here).  This is not the best
+   way to require that the user's domain matches the URL and is a 
+   big hassle for users most of the time, so it is not a recommended 
+   option.  If you are looking for a way to ensure the domain on the 
+   username matches the URL, use $removeDomainIfGiven and make sure 
+   $dontUseHostName is turned off.  This option is mostly useful if 
+   you have a combination of usernames both WITH and WITHOUT (in 
+   combination with $removeDomainAfterDomainCheck) hostname portions 
+   and are thus using $dontUseHostName, but you still want to make 
+   sure the username's domain portion matches the URL.
+
+   $removeDomainAfterDomainCheck
+   -----------------------------
+   will remove the domain portion of the username if given.  This 
+   option should not be used in favor of $removeDomainIfGiven if at
+   all possible.  This setting is usually only used when 
+   $forceDomainInUsername is turned on so usernames that shouldn't
+   have domain portions on them...............
+
+DUH.  Took me this long to figure out that I was spinnin my wheels for
+a user who was asking the impossible.  This just here for posterity...
+*/
+
+
+
+  // check and see if they decided to insert the host anyway
   // or we don't want to use the host name...
   //
-  if( !$dontUseHostName && !strstr( $user, $at ) ) { 
+  if( $alwaysAddHostName || (!$dontUseHostName && strpos($user, $at) === FALSE) ) {
 
 
     $hostname = deconstructDomainName($hostname);
@@ -470,126 +549,112 @@ function vlogin_domain_do() {
      // was confusing to people using $dontUseHostName 
      // echo '<br>$hostname is ' . $hostname . '<br>';
      echo '<br>PHP_SELF is ' . $_SERVER['PHP_SELF'] . '<br>';
+global        $imapServerAddress;
+echo '<hr>' . $imapServerAddress . '<hr>';
      echo '<br><hr></body></html>';
      exit;
 
   }
 
-
-  // look for users trying to log into a domain that is locked out
-  //
-  if ($lockoutEnabled)
-  {
-
-     if (strpos($realname, $at) !== FALSE)
-     {
-
-        $usersDomain = substr($realname, strpos($realname, $at) + 1);
-
-        foreach (array_keys($virtualDomains) as $virtualDomain) {
-
-          if (stristr($usersDomain, $virtualDomain)) {
-
-              if ($virtualDomains[$virtualDomain]['###LOCKED OUT###'])
-              {
-                 sqsession_destroy();
-                 header('Location: ../plugins/vlogin/' 
-                    . $virtualDomains[$virtualDomain]['###LOCKED OUT###']);
-                 exit;
-              }
-
-          }
-
-        }
-
-     }
-     else
-        $usersDomain = '';
-
-
-     if ($lockoutTable)
-     {
-        check_lockouts($lockoutTable, $realname, $usersDomain);
-     }
-
-  }
-
-
 }
 
 
 
-function perUserOverride($user) {
+/** 
+  * Get Per-User Config Overrides
+  *
+  * Looks up a user in the per-user config file and, if found,
+  * grabs and uses the user's custom settings.
+  *
+  * @param string $user The username for which to look for custom settings.
+  *
+  */
+function perUserOverride($user) 
+{
 
-  global $at, $atConversion, $perUserSettingsFile, $useSessionBased;
+   global $at, $atConversion, $perUserSettingsFile, $useSessionBased,
+          $plugins, $squirrelmail_plugin_hooks;
 
 
-  // convert "at" sign...
-  //
-  foreach ($atConversion as $otherAt)
-     $user = str_replace($otherAt, $at, $user);
+   // convert "at" sign...
+   //
+   foreach ($atConversion as $otherAt)
+      $user = str_replace($otherAt, $at, $user);
 
 
-  // override per-user settings
-  //
-  if (!empty($perUserSettingsFile) && !empty($user)) {
+   // override per-user settings
+   //
+   if (!empty($perUserSettingsFile) && !empty($user)) {
 
 
-     // find settings for user
-     //
-     $userSettings = array();
-     if ($USERSETTINGS = @fopen ($perUserSettingsFile, "r"))
-     {
+      // find settings for user
+      //
+      $userSettings = array();
+      if ($USERSETTINGS = @fopen ($perUserSettingsFile, "r"))
+      {
 
-        while (!feof($USERSETTINGS))
+         while (!feof($USERSETTINGS))
+         {
+
+            $line = fgets($USERSETTINGS, 4096);
+            $line = trim($line);
+
+
+            // skip blank lines and comment lines
+            //
+            if (strpos($line, '#') === 0 || strlen($line) < 3)
+               continue;
+
+
+            // parse fields out
+            //
+            if (substr($line, strlen($line) - 1) != ',') $line .= ',';
+            preg_match_all('/(.+?),\s*/', $line, $configSettings, PREG_PATTERN_ORDER);
+
+
+            // stop when we have the right username (case insensitive)
+            //
+            if (strtoupper($user) == strtoupper($configSettings[1][0]))
+            {
+               $userSettings = $configSettings[1];
+               break;
+            }
+
+
+            // if wildcard match for user is found, 
+            // grab settings, but don't stop looking 
+            // for exact match
+            //
+            if (strpos($configSettings[1][0], '*') !== FALSE
+             || strpos($configSettings[1][0], '?') !== FALSE)
+            {
+               if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'), 
+                           strtoupper($configSettings[1][0])) . '$/', strtoupper($user)))
+                  $userSettings = $configSettings[1];
+            }
+
+         }
+
+         fclose($USERSETTINGS);
+
+      }
+
+      if (sizeof($userSettings) > 0)
+      {
+
+
+        // put arrays of values back together
+        //
+        for ($x = 0; $x < sizeof($userSettings); $x++)
         {
-
-           $line = fgets($USERSETTINGS, 4096);
-           $line = trim($line);
-
-
-           // skip blank lines and comment lines
-           //
-           if (strpos($line, '#') === 0 || strlen($line) < 3)
-              continue;
-
-
-           // parse fields out
-           //
-           if (substr($line, strlen($line) - 1) != ',') $line .= ',';
-           preg_match_all('/(.+?),\s*/', $line, $configSettings, PREG_PATTERN_ORDER);
-
-
-           // stop when we have the right username (case insensitive)
-           //
-           if (strtoupper($user) == strtoupper($configSettings[1][0]))
+           if (substr($userSettings[$x], 0, 1) == ',')
            {
-              $userSettings = $configSettings[1];
-              break;
+              $userSettings[$x - 1] .= $userSettings[$x];
+              array_splice($userSettings, $x, 1);
+              $x--;
            }
-
-
-           // if wildcard match for user is found, 
-           // grab settings, but don't stop looking 
-           // for exact match
-           //
-           if (strpos($configSettings[1][0], '*') !== FALSE
-            || strpos($configSettings[1][0], '?') !== FALSE)
-           {
-              if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'), 
-                          strtoupper($configSettings[1][0])) . '$/', strtoupper($user)))
-                 $userSettings = $configSettings[1];
-           }
-
         }
 
-        fclose($USERSETTINGS);
-
-     }
-
-     if (sizeof($userSettings) > 0)
-     {
-
 
         if ($useSessionBased)
         {
@@ -611,9 +676,20 @@ function perUserOverride($user) {
               continue;
 
            $matches = explode('=', $setting);
+
+
+           // convert strings "true" and "false" to real boolean values
+           //
+           if (strtoupper($matches[1]) === 'TRUE') $matches[1] = TRUE;
+           else if (strtoupper($matches[1]) === 'FALSE') $matches[1] = FALSE;
+
+
            if ($useSessionBased)
            {
-              $config_override[$matches[0]] = $matches[1];
+              if (stristr($matches[0], 'enable_plugins') || stristr($setting, 'disable_plugins'))
+                 $config_override[$matches[0]] = explode(',', $matches[1]);
+              else
+                 $config_override[$matches[0]] = $matches[1];
            }
            else
            {
@@ -628,11 +704,49 @@ function perUserOverride($user) {
                  global $$matches[0];
                  eval('$$matches[0] = ' . $matches[1] . ';');
               }
+
+
+              // enable additional plugins
+              //
+              else if (stristr($matches[0], 'enable_plugins'))
+              {
+                 $values = explode(',', $matches[1]);
+                 foreach ($values as $pluginName)
+                 {
+                    $plugins[] = $pluginName;
+                    use_plugin($pluginName);
+                 }
+              }
+
+
+              // disable plugins
+              //
+              else if (stristr($setting, 'disable_plugins'))
+              {
+                 $values = explode(',', $matches[1]);
+                 foreach ($values as $pluginName)
+                 {
+                    $pluginKey = array_search($pluginName, $plugins);
+                    if (!is_null($pluginKey) && $pluginKey !== FALSE)
+                    {
+                       unset($plugins[$pluginKey]);
+                       foreach (array_keys($squirrelmail_plugin_hooks) as $hookName)
+                       {
+                          unset($squirrelmail_plugin_hooks[$hookName][$pluginName]);
+                       }
+                    }
+                 }
+              }
+
+
+              // regular config setting...
+              //
               else
               {
                  global $$matches[0];
                  $$matches[0] = $matches[1];
               }
+
            }
 
         }
@@ -643,10 +757,7 @@ function perUserOverride($user) {
       
            compatibility_sqsession_register($config_override, 'config_override');
       
-           if (compatibility_check_sm_version(1, 3))
-              include_once (SM_PATH . 'plugins/multilogin/functions.php');
-           else
-              include_once ('../plugins/multilogin/functions.php');
+           include_once (SM_PATH . 'plugins/multilogin/functions.php');
 
            multilogin_sqoverride_config_do();
 
@@ -660,23 +771,51 @@ function perUserOverride($user) {
 
 
 
-// Does the actual hostname deconstruction
-//
-function deconstructDomainName($hostname) {
+/** 
+  * Host Name Deconstruction
+  *
+  * Takes a host name (usually from URL, but can be
+  * anything) and reconstructs it based on vlogin
+  * configuration settings.
+  *
+  * @param string $hostname The hostname to re/deconstruct.
+  *
+  * @return string The rebuilt hostname.
+  *
+  */
+function deconstructDomainName($hostname) 
+{
 
   global $notPartOfDomainName, $chopOffDotSectionsFromRight, 
          $chopOffDotSectionsFromLeft, $numberOfDotSections, 
-         $checkByExcludeList, $at, $dot, $dontUseHostName,
-         $removeFromFront, $translateHostnameTable, $pathToQmail;
+         $checkByExcludeList, $at, $dot, $replacements,
+         $removeFromFront, $translateHostnameTable, $pathToQmail,
+         $reverseDotSectionOrder;
 
 
-  if (compatibility_check_sm_version(1, 3))
-     include_once (SM_PATH . 'plugins/vlogin/data/config.php');
-  else
-     include_once ('../plugins/vlogin/data/config.php');
+  include_once (SM_PATH . 'plugins/vlogin/data/config.php');
 
 
 
+  // if a port number is in the URL, remove it before proceeding
+  //
+  $hostname = preg_replace('/:\d+/', '', $hostname);
+
+
+
+  // replace any characters as necessary
+  //
+  if (is_array($replacements))
+    foreach ($replacements as $chars => $repl)
+      $hostname = str_replace($chars, $repl, $hostname);
+
+
+  // reverse dot section order if necessary
+  //
+  if ($reverseDotSectionOrder)
+    $hostname = implode('.', array_reverse(explode('.', $hostname))); 
+
+
   // if enabled, remove "dot sections" until desired size of hostname is reached
   //
   if ($numberOfDotSections > 0) {
@@ -714,6 +853,10 @@ function deconstructDomainName($hostname) {
   }
 
 
+// TODO: I think it makes more sense to do this BEFORE 
+// doing the $numberOfDotSections functionality.  I would
+// move it now, but I don't know if I'll break things for
+// people out there.... ;>
    // if enabled, extract any of the undesired host name pieces
    //
    if ($checkByExcludeList) {
@@ -722,7 +865,7 @@ function deconstructDomainName($hostname) {
       {
 
          $hostname = preg_replace('/(^|\.)' . $dotSection . '($|\.)/', 
-                                  '${1}', $hostname);
+                                  "\$1", $hostname);
 
       }
 
@@ -759,8 +902,15 @@ function deconstructDomainName($hostname) {
 
 
 
-// unalias any Qmail/Vpopmail domain aliases
-//
+/**
+  * Unalias any Qmail/Vpopmail Domain Aliases
+  *
+  * @param string $host Host name to be dealiased.
+  * @param string $pathToQmail System path to Qmail install directory.
+  *
+  * @return string Corrected host name.
+  *
+  */
 function unaliasQmailDomainAlias($host, $pathToQmail)
 {
     $tmp = '+'.$host.'-:' ;
@@ -784,6 +934,19 @@ function unaliasQmailDomainAlias($host, $pathToQmail)
 // remap a host name to the one specified in the translate host 
 // name table (path given by the $translateHostnameTable)
 //
+/**
+  * Remap Host Name
+  *
+  * Takes a host name, looks it up in the given lookup table,
+  * and, if found therein, returns the lookup value (otherwise,
+  * returns the host as is).
+  *
+  * @param string $host The host name to be looked up.
+  * @param string $translateHostnameTable The path to the lookup table.
+  *
+  * @return string The remapped host name.
+  *
+  */
 function translateHostname($host, $translateHostnameTable)
 {
 
@@ -829,31 +992,27 @@ function translateHostname($host, $translateHostnameTable)
    }
 
 
-   // if we got here and a catchall login is available, use it
+   // otherwise, unaltered hostname is returned
    //
-   if (!empty($catchallLogin))
-   {
-
-      // remove extraneous stuff we don't need off end (begins with plus sign)
-      //
-      $catchallLogin = preg_replace('/\+%\S+\s*$/', '', $catchallLogin);
-
-      return $catchallLogin;
-
-   }
-
-
-   // otherwise, unaltered username is returned
-   //
-   return $user;
+   return $host;
 
 }
 
 
 
-// remap a user login to the one specified in the sendmail virtual
-// user table (path given by the $sendmailVirtualUserTable parameter)
-//
+/**
+  * Remap User Login ala Sendmail Style Virtual User Table
+  *
+  * Looks up the given username in the given virtual user table
+  * and, if found therein, returns the corresponding lookup value
+  * (otherwise, returns the username as is).
+  *
+  * @param string $user The username to be looked up.
+  * @param string $sendmailVirtualUserTable The system path to the lookup table.
+  *
+  * @return string The remapped user name.
+  *
+  */
 function getSendmailVirtualUser($user, $sendmailVirtualUserTable)
 {
 
@@ -935,73 +1094,4 @@ function getSendmailVirtualUser($user, $sendmailVirtualUserTable)
 
 
 
-// parses a table that lists domains and/or users that are
-// to be locked out and not allowed to log in
-//
-function check_lockouts($lockoutTable, $user, $host)
-{
-
-   if ($LOCKOUTTABLE = @fopen ($lockoutTable, 'r'))
-   {
-
-      while (!feof($LOCKOUTTABLE))
-      {
-
-         $line = fgets($LOCKOUTTABLE, 4096);
-         $line = trim($line);
-
-
-         // skip blank lines and comment lines
-         //
-         if (strpos($line, '#') === 0 || strlen($line) < 3)
-            continue;
-
-
-         // if we were given a hostname, see if this is 
-         // a domain lockout line
-         //
-         if (!empty($host) && preg_match('/^\s*domain:\s*(\S+)\s+(.+)\s*$/', $line, $matches))
-         {
-
-            // check for match with hostname, redirect if found
-            //
-            if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'), 
-                          strtoupper($matches[1])) . '$/', strtoupper($host)))
-            {
-               fclose($LOCKOUTTABLE);
-               sqsession_destroy();
-               header('Location: ../plugins/vlogin/' . $matches[2]);
-               exit;
-            }
-
-         }
-
-
-         // if we were given a username, see if this is 
-         // a user lockout line
-         //
-         if (!empty($user) && preg_match('/^\s*user:\s*(\S+)\s+(.+)\s*$/', $line, $matches))
-         {
-
-            // check for match with username, redirect if found
-            //
-            if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'), 
-                          strtoupper($matches[1])) . '$/', strtoupper($user)))
-            {
-               fclose($LOCKOUTTABLE);
-               sqsession_destroy();
-               header('Location: ../plugins/vlogin/' . $matches[2]);
-               exit;
-            }
-
-         }
-
-      }
-
-      fclose($LOCKOUTTABLE);
-
-   }
-}
-
-
 ?>
